@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2019 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2020 -- leonerd@leonerd.org.uk
 
 package IO::Async::Loop;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.73';
+our $VERSION = '0.76';
 
 # When editing this value don't forget to update the docs below
 use constant NEED_API_VERSION => '0.33';
@@ -30,13 +30,13 @@ use constant WATCHDOG_SIGABRT  => $ENV{IO_ASYNC_WATCHDOG_SIGABRT};
 
 use Carp;
 
-use IO::Socket (); # empty import
 use Time::HiRes qw(); # empty import
 use POSIX qw( WNOHANG );
 use Scalar::Util qw( refaddr weaken );
 use Socket qw( SO_REUSEADDR AF_INET6 IPPROTO_IPV6 IPV6_V6ONLY );
 
 use IO::Async::OS;
+use IO::Async::Metrics '$METRICS';
 
 use constant HAVE_SIGNALS => IO::Async::OS->HAVE_SIGNALS;
 use constant HAVE_POSIX_FORK => IO::Async::OS->HAVE_POSIX_FORK;
@@ -209,7 +209,7 @@ no class was successful, then an exception is thrown.
 The constructed object is cached, and will be returned again by a subsequent
 call. The cache will also be set by a constructor on a specific subclass. This
 behaviour makes it possible to simply use the normal constructor in a module
-that wishes to interract with the main program's Loop, such as an integration
+that wishes to interact with the main program's Loop, such as an integration
 module for another event system.
 
 For example, the following two C<$loop> variables will refer to the same
@@ -403,6 +403,7 @@ sub _add_noparentcheck
    my $nkey = refaddr $notifier;
 
    $self->{notifiers}->{$nkey} = $notifier;
+   $METRICS and $METRICS->inc_gauge( notifiers => );
 
    $notifier->__set_loop( $self );
 
@@ -442,6 +443,7 @@ sub _remove_noparentcheck
    exists $self->{notifiers}->{$nkey} or croak "Notifier does not exist in collection";
 
    delete $self->{notifiers}->{$nkey};
+   $METRICS and $METRICS->dec_gauge( notifiers => );
 
    $notifier->__set_loop( undef );
 
@@ -594,7 +596,9 @@ need recreating in the new child process before the program can continue.
 
 sub post_fork
 {
-   # empty
+   my $self = shift;
+
+   IO::Async::OS->post_fork( $self );
 }
 
 ###########
@@ -773,7 +777,7 @@ A CODE reference to the handling callback.
 =back
 
 Attaching to C<SIGCHLD> is not recommended because of the way all child
-processes use it to report their termination. Instead, the C<watch_child>
+processes use it to report their termination. Instead, the C<watch_process>
 method should be used to watch for termination of a given child process. A
 warning will be printed if C<SIGCHLD> is passed here, but in future versions
 of L<IO::Async> this behaviour may be disallowed altogether.
@@ -793,9 +797,9 @@ sub attach_signal
    HAVE_SIGNALS or croak "This OS cannot ->attach_signal";
 
    if( $signal eq "CHLD" ) {
-      # We make special exception to allow $self->watch_child to do this
+      # We make special exception to allow $self->watch_process to do this
       caller eq "IO::Async::Loop" or
-         carp "Attaching to SIGCHLD is not advised - use ->watch_child instead";
+         carp "Attaching to SIGCHLD is not advised - use ->watch_process instead";
    }
 
    if( not $self->{sigattaches}->{$signal} ) {
@@ -1211,6 +1215,20 @@ Optional reference to an array giving a list of names of values which should
 be returned by resolving future. Values will be returned in the same order as
 in the list. Valid choices are: C<exitcode>, C<stdout>, C<stderr>.
 
+=item cancel_signal => STRING
+
+Optional. Name (or number) of the signal to send to the process if the
+returned future is cancelled. Defaults to C<TERM>. Use empty string or zero
+disable sending a signal on cancellation.
+
+=item fail_on_nonzero => BOOL
+
+Optional. If true, the returned future will fail if the process exits with a
+nonzero status. The failure will contain a message, the C<process> category
+name, and the capture values that were requested.
+
+   Future->fail( $message, process => @captures )
+
 =back
 
 This method is intended mainly as an IO::Async-compatible replacement for the
@@ -1253,9 +1271,9 @@ sub _run_process
       $subparams{stdin} = { from => $child_stdin };
    }
 
-   $subparams{code}    = delete $params{code};
-   $subparams{command} = delete $params{command};
-   $subparams{setup}   = delete $params{setup};
+   foreach (qw( code command setup notifier_name )) {
+      $subparams{$_} = delete $params{$_};
+   }
 
    foreach my $name ( @$capture ) {
       grep { $_ eq $name } qw( exitcode stdout stderr ) or croak "Unexpected capture $name";
@@ -1264,6 +1282,10 @@ sub _run_process
       $subparams{stderr} = { into => \$results{stderr} } if $name eq "stderr";
    }
 
+   my $cancel_signal = delete $params{cancel_signal} // "TERM";
+
+   my $fail_on_nonzero = delete $params{fail_on_nonzero};
+
    croak "Unrecognised parameters " . join( ", ", keys %params ) if keys %params;
 
    my $future = $self->new_future;
@@ -1274,10 +1296,21 @@ sub _run_process
       on_finish => sub {
          ( undef, $results{exitcode} ) = @_;
 
-         $future->done( @results{ @$capture } );
+         if( $fail_on_nonzero and $results{exitcode} > 0 ) {
+            $future->fail( "Process failed with exit code $results{exitcode}\n",
+               process => @results{ @$capture }
+            );
+         }
+         else {
+            $future->done( @results{ @$capture } );
+         }
       },
    );
 
+   $future->on_cancel(sub {
+      $process->kill( $cancel_signal );
+   }) if $cancel_signal;
+
    $self->add( $process );
 
    return ( $future, $process );
@@ -1316,7 +1349,7 @@ The second argument is passed the plain perl C<$?> value.
 
 =back
 
-This method should not be used in new code; intead use L</run_process>
+This method should not be used in new code; instead use L</run_process>
 directly.
 
 =cut
@@ -2011,6 +2044,8 @@ sub _listen_addrs
          next;
       }
 
+      $sock->blocking( 0 );
+
       if( $reuseaddr ) {
          unless( $sock->sockopt( SO_REUSEADDR, 1 ) ) {
             $sockopterr = $!;
@@ -2136,7 +2171,7 @@ be invoked in the following way:
 The second argument is passed the plain perl C<$?> value.
 
 This key is optional; if not supplied, the calling code should install a
-handler using the C<watch_child> method.
+handler using the C<watch_process> method.
 
 =item keep_signals => BOOL
 
@@ -2180,9 +2215,11 @@ sub fork
    }
 
    if( defined $params{on_exit} ) {
-      $self->watch_child( $kid => $params{on_exit} );
+      $self->watch_process( $kid => $params{on_exit} );
    }
 
+   $METRICS and $METRICS->inc_counter( forks => );
+
    return $kid;
 }
 
@@ -2248,6 +2285,7 @@ sub create_thread
    unless( $self->{thread_join_pipe} ) {
       ( my $rd, $self->{thread_join_pipe} ) = IO::Async::OS->pipepair or
          croak "Cannot pipepair - $!";
+      $rd->blocking( 0 );
       $self->{thread_join_pipe}->autoflush(1);
 
       $self->watch_io(
@@ -2327,6 +2365,19 @@ This method may be implemented using C<constant>; e.g
 
 =cut
 
+sub pre_wait
+{
+   my $self = shift;
+   $METRICS and $self->{processing_start} and
+      $METRICS->inc_distribution_by( processing_time => Time::HiRes::tv_interval $self->{processing_start} );
+}
+
+sub post_wait
+{
+   my $self = shift;
+   $METRICS and $self->{processing_start} = [ Time::HiRes::gettimeofday ];
+}
+
 =head2 watch_io
 
    $loop->watch_io( %params )
@@ -2844,9 +2895,9 @@ sub _reap_children
    }
 }
 
-=head2 watch_child
+=head2 watch_process
 
-   $loop->watch_child( $pid, $code )
+   $loop->watch_process( $pid, $code )
 
 This method adds a new handler for the termination of the given child process
 PID, or all child processes.
@@ -2869,9 +2920,9 @@ The second argument is passed the plain perl C<$?> value.
 
 After invocation, the handler for a PID-specific watch is automatically
 removed. The all-child watch will remain until it is removed by
-C<unwatch_child>.
+C<unwatch_process>.
 
-This and C<unwatch_child> are optional; a subclass may implement neither, or
+This and C<unwatch_process> are optional; a subclass may implement neither, or
 both. If it implements neither then child watching will be performed by using
 C<watch_signal> to install a C<SIGCHLD> handler, which will use C<waitpid> to
 look for exited child processes.
@@ -2881,11 +2932,17 @@ ordering guarantee as to which will be called first.
 
 =cut
 
-sub watch_child
+sub watch_process
 {
    my $self = shift;
    my ( $pid, $code ) = @_;
 
+   if( $self->API_VERSION < 0.76 and
+      ( $self->can( "watch_child" ) // 0 ) != \&watch_child ) {
+      # Invoke legacy loop API
+      return $self->watch_child( @_ );
+   }
+
    my $childwatches = $self->{childwatches};
 
    croak "Already have a handler for $pid" if exists $childwatches->{$pid};
@@ -2907,19 +2964,28 @@ sub watch_child
    $childwatches->{$pid} = $code;
 }
 
-=head2 unwatch_child
+# Old name
+sub watch_child { shift->watch_process( @_ ) }
 
-   $loop->unwatch_child( $pid )
+=head2 unwatch_process
+
+   $loop->unwatch_process( $pid )
 
 This method removes a watch on an existing child process PID.
 
 =cut
 
-sub unwatch_child
+sub unwatch_process
 {
    my $self = shift;
    my ( $pid ) = @_;
 
+   if( $self->API_VERSION < 0.76 and
+      ( $self->can( "unwatch_child" ) // 0 ) != \&unwatch_child ) {
+      # Invoke legacy loop API
+      return $self->unwatch_child( @_ );
+   }
+
    my $childwatches = $self->{childwatches};
 
    delete $childwatches->{$pid};
@@ -2929,6 +2995,9 @@ sub unwatch_child
    }
 }
 
+# Old name
+sub unwatch_child { shift->unwatch_process( @_ ) }
+
 =head1 METHODS FOR SUBCLASSES
 
 The following methods are provided to access internal features which are
