@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2006-2018 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2006-2020 -- leonerd@leonerd.org.uk
 
 package IO::Async::Stream;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.74';
+our $VERSION = '0.76';
 
 use base qw( IO::Async::Handle );
 
@@ -20,6 +20,7 @@ use Encode 2.11 qw( find_encoding STOP_AT_PARTIAL );
 use Scalar::Util qw( blessed );
 
 use IO::Async::Debug;
+use IO::Async::Metrics '$METRICS';
 
 # Tuneable from outside
 # Not yet documented
@@ -821,6 +822,7 @@ sub _flush_one_write
       $self->debug_printf( "WRITE err=%d/%s", $errno, $errno ) if $IO::Async::Debug::DEBUG > 1;
 
       if( $errno == EPIPE ) {
+         $self->debug_printf( "WRITE-EOF" );
          $self->{write_eof} = 1;
          $self->maybe_invoke_event( on_write_eof => );
       }
@@ -832,6 +834,8 @@ sub _flush_one_write
       return 0;
    }
 
+   $METRICS and $METRICS->inc_counter_by( stream_written => $len );
+
    if( my $on_write = $head->on_write ) {
       $on_write->( $self, $len );
    }
@@ -1026,6 +1030,8 @@ sub _do_read
          IO::Async::Debug::log_hexdump( $data ) if $IO::Async::Debug::DEBUG_FLAGS{Sr};
       }
 
+      $METRICS and $METRICS->inc_counter_by( stream_read => $len );
+
       my $eof = $self->{read_eof} = ( $len == 0 );
 
       if( my $encoding = $self->{encoding} ) {
@@ -1039,6 +1045,7 @@ sub _do_read
       1 while $self->_flush_one_read( $eof );
 
       if( $eof ) {
+         $self->debug_printf( "READ-EOF" );
          $self->maybe_invoke_event( on_read_eof => );
          $self->close_now if $self->{close_on_read_eof};
          foreach ( @{ $self->{readqueue} } ) {
